@@ -1,6 +1,6 @@
 /*
  * cups - Python bindings for CUPS
- * Copyright (C) 2002, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013  Red Hat, Inc
+ * Copyright (C) 2002, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015  Red Hat, Inc
  * Author: Tim Waugh <twaugh@redhat.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -52,7 +52,7 @@ static pthread_once_t tls_key_once = PTHREAD_ONCE_INIT;
 # define IPP_PKI_ERROR			0x1001
 #endif /* CUPS < 1.5 */
 
-#if (CUPS_VERSION_MAJOR == 1 && CUPS_VERSION_MINOR > 5)
+#if HAVE_CUPS_1_6
 # define CUPS_SERVER_REMOTE_PRINTERS	"_remote_printers"
 #endif /* CUPS > 1.5 */
 
@@ -273,6 +273,12 @@ ippGetResolution(
   return (attr->values[element].resolution.xres);
 }
 
+ipp_state_t
+ippGetState(ipp_t *ipp)
+{
+  return (ipp->state);
+}
+
 ipp_status_t
 ippGetStatusCode(ipp_t *ipp)
 {
@@ -327,6 +333,22 @@ ippSetOperation(ipp_t    *ipp,
   return (1);
 }
 
+int
+ippSetState(ipp_t       *ipp,
+	    ipp_state_t state)
+{
+  ipp->state = state;
+  return (1);
+}
+
+int
+ippSetStatusCode(ipp_t        *ipp,
+		 ipp_status_t status)
+{
+  ipp->request.status.status_code = status;
+  return (1);
+}
+
 int
 ippSetString(ipp_t           *ipp,
              ipp_attribute_t **attr,
@@ -732,6 +754,28 @@ cups_connectDest (PyObject *self, PyObject *args, PyObject *kwds)
 }
 #endif /* HAVE_CUPS_1_6 */
 
+static PyObject *
+cups_ippErrorString (PyObject *self, PyObject *args)
+{
+  int op;
+
+  if (!PyArg_ParseTuple (args, "i", &op))
+    return NULL;
+
+  return PyUnicode_FromString (ippErrorString (op));
+}
+
+static PyObject *
+cups_ippOpString (PyObject *self, PyObject *args)
+{
+  int op;
+
+  if (!PyArg_ParseTuple (args, "i", &op))
+    return NULL;
+
+  return PyUnicode_FromString (ippOpString (op));
+}
+
 static PyObject *
 cups_require (PyObject *self, PyObject *args)
 {
@@ -903,6 +947,20 @@ static PyMethodDef cups_methods[] = {
     "@return: a 2-tuple of the Connection object and the HTTP resource.\n"},
 #endif /* HAVE_CUPS_1_6 */
 
+  { "ippErrorString",
+    (PyCFunction) cups_ippErrorString, METH_VARARGS,
+    "ippErrorString(statuscode) -> name\n\n"
+    "@type statuscode: integer\n"
+    "@param statuscode: IPP Request status code\n"
+    "@return: a string describing the status code\n"},
+
+  { "ippOpString",
+    (PyCFunction) cups_ippOpString, METH_VARARGS,
+    "ippOpString(op) -> name\n\n"
+    "@type op: integer\n"
+    "@param op: IPP Request operation\n"
+    "@return: a string representing the operation name\n"},
+
   { "require", cups_require, METH_VARARGS,
     "require(version) -> None\n\n"
     "Require pycups version.\n\n"
@@ -1042,15 +1100,37 @@ initcups (void)
 
   // Constants
 #if PY_MAJOR_VERSION >= 3
-  #define INT_CONSTANT(name)					\
+#  define INT_CONSTANT(name)					\
     PyDict_SetItemString (d, #name, PyLong_FromLong (name))
+#  define INT_CONSTANT_AS(name,alias)				\
+    PyDict_SetItemString (d, alias, PyLong_FromLong (name))
+#  define INT_CONSTANT_ALIAS(name,alias)			\
+    PyDict_SetItemString (d, #name, PyLong_FromLong (name));	\
+    PyDict_SetItemString (d, alias, PyLong_FromLong (name))
 #else
-#define INT_CONSTANT(name)					\
+#  define INT_CONSTANT(name)					\
   PyDict_SetItemString (d, #name, PyInt_FromLong (name))
+#  define INT_CONSTANT_AS(name,alias)				\
+  PyDict_SetItemString (d, alias, PyInt_FromLong (name))
+#  define INT_CONSTANT_ALIAS(name,alias)				\
+  PyDict_SetItemString (d, #name, PyInt_FromLong (name));	\
+  PyDict_SetItemString (d, alias, PyInt_FromLong (name))
 #endif
 #define STR_CONSTANT(name)					\
   PyDict_SetItemString (d, #name, PyUnicode_FromString (name))
 
+#if CUPS_VERSION_MAJOR > 1 || (CUPS_VERSION_MAJOR == 1 && CUPS_VERSION_MINOR >= 6)
+#  define INT_16_CONSTANT(newname,oldname)	\
+  INT_CONSTANT_ALIAS(newname,#oldname)
+#  define INT_16_CONSTANT_NEWNAME(newname,oldname)	\
+  INT_CONSTANT(newname)
+#else /* CUPS < 1.5 */
+#  define INT_16_CONSTANT(newname,oldname)	\
+  INT_CONSTANT_ALIAS(oldname,#newname)
+#  define INT_16_CONSTANT_NEWNAME(newname,oldname)	\
+  INT_CONSTANT_AS(oldname,#newname)
+#endif /* CUPS < 1.5 */
+
   // CUPS printer types
   INT_CONSTANT (CUPS_PRINTER_LOCAL);
   INT_CONSTANT (CUPS_PRINTER_CLASS);
@@ -1095,23 +1175,33 @@ initcups (void)
   STR_CONSTANT (CUPS_FORMAT_TEXT);
 
   // Selected HTTP status codes
-  INT_CONSTANT (HTTP_ERROR);
-  INT_CONSTANT (HTTP_OK);
-  INT_CONSTANT (HTTP_NOT_MODIFIED);
-  INT_CONSTANT (HTTP_BAD_REQUEST);
-  INT_CONSTANT (HTTP_UNAUTHORIZED);
-  INT_CONSTANT (HTTP_FORBIDDEN);
-  INT_CONSTANT (HTTP_NOT_FOUND);
-  INT_CONSTANT (HTTP_REQUEST_TIMEOUT);
-  INT_CONSTANT (HTTP_UPGRADE_REQUIRED);
-  INT_CONSTANT (HTTP_SERVER_ERROR);
-  INT_CONSTANT (HTTP_NOT_IMPLEMENTED);
-  INT_CONSTANT (HTTP_BAD_GATEWAY);
-  INT_CONSTANT (HTTP_SERVICE_UNAVAILABLE);
-  INT_CONSTANT (HTTP_GATEWAY_TIMEOUT);
-  INT_CONSTANT (HTTP_NOT_SUPPORTED);
-  INT_CONSTANT (HTTP_AUTHORIZATION_CANCELED);
-  INT_CONSTANT (HTTP_PKI_ERROR);
+  /* Also define legacy names */
+#if CUPS_VERSION_MAJOR > 1 || (CUPS_VERSION_MAJOR == 1 && CUPS_VERSION_MINOR >= 6)
+#  define INT_HTTP_STATUS_CONSTANT(name)		\
+  INT_CONSTANT_ALIAS(HTTP_STATUS_##name, "HTTP_"#name)
+#else /* CUPS < 1.6 */
+#  define INT_HTTP_STATUS_CONSTANT(name)		\
+  INT_CONSTANT_ALIAS(HTTP_##name, "HTTP_STATUS_"#name)
+#endif /* CUPS < 1.6 */
+
+  INT_HTTP_STATUS_CONSTANT (ERROR);
+  INT_HTTP_STATUS_CONSTANT (OK);
+  INT_HTTP_STATUS_CONSTANT (NOT_MODIFIED);
+  INT_HTTP_STATUS_CONSTANT (BAD_REQUEST);
+  INT_HTTP_STATUS_CONSTANT (UNAUTHORIZED);
+  INT_HTTP_STATUS_CONSTANT (FORBIDDEN);
+  INT_HTTP_STATUS_CONSTANT (NOT_FOUND);
+  INT_HTTP_STATUS_CONSTANT (REQUEST_TIMEOUT);
+  INT_HTTP_STATUS_CONSTANT (UPGRADE_REQUIRED);
+  INT_HTTP_STATUS_CONSTANT (SERVER_ERROR);
+  INT_HTTP_STATUS_CONSTANT (NOT_IMPLEMENTED);
+  INT_HTTP_STATUS_CONSTANT (BAD_GATEWAY);
+  INT_HTTP_STATUS_CONSTANT (SERVICE_UNAVAILABLE);
+  INT_HTTP_STATUS_CONSTANT (GATEWAY_TIMEOUT);
+  INT_HTTP_STATUS_CONSTANT (NOT_SUPPORTED);
+  INT_16_CONSTANT (HTTP_STATUS_CUPS_AUTHORIZATION_CANCELED,
+		   HTTP_AUTHORIZATION_CANCELED);
+  INT_16_CONSTANT (HTTP_STATUS_CUPS_PKI_ERROR, HTTP_PKI_ERROR);
 
   // PPD UI enum
   INT_CONSTANT (PPD_UI_BOOLEAN);
@@ -1119,7 +1209,6 @@ initcups (void)
   INT_CONSTANT (PPD_UI_PICKMANY);
 
   // PPD Order dependency enum
-
   INT_CONSTANT (PPD_ORDER_ANY);
   INT_CONSTANT (PPD_ORDER_DOCUMENT);
   INT_CONSTANT (PPD_ORDER_EXIT);
@@ -1176,10 +1265,19 @@ initcups (void)
   INT_CONSTANT (IPP_FINISHINGS_BIND_BOTTOM);
 
   // IPP orientations
-  INT_CONSTANT (IPP_PORTRAIT);
-  INT_CONSTANT (IPP_LANDSCAPE);
-  INT_CONSTANT (IPP_REVERSE_PORTRAIT);
-  INT_CONSTANT (IPP_REVERSE_LANDSCAPE);
+  /* Also define legacy names */
+#if CUPS_VERSION_MAJOR > 1 || (CUPS_VERSION_MAJOR == 1 && CUPS_VERSION_MINOR >= 6)
+#  define INT_IPP_ORIENT_CONSTANT(name)	\
+  INT_CONSTANT_ALIAS(IPP_ORIENT_##name, "IPP_"#name)
+#else /* CUPS < 1.6 */
+#  define INT_IPP_ORIENT_CONSTANT(name)		\
+  INT_CONSTANT_ALIAS(IPP_##name, "IPP_ORIENT_"#name)
+#endif /* CUPS < 1.6 */
+
+  INT_IPP_ORIENT_CONSTANT (PORTRAIT);
+  INT_IPP_ORIENT_CONSTANT (LANDSCAPE);
+  INT_IPP_ORIENT_CONSTANT (REVERSE_PORTRAIT);
+  INT_IPP_ORIENT_CONSTANT (REVERSE_LANDSCAPE);
 
   // IPP qualities
   INT_CONSTANT (IPP_QUALITY_DRAFT);
@@ -1187,59 +1285,88 @@ initcups (void)
   INT_CONSTANT (IPP_QUALITY_HIGH);
 
   // IPP errors
-  INT_CONSTANT (IPP_OK);
-  INT_CONSTANT (IPP_OK_SUBST);
-  INT_CONSTANT (IPP_OK_CONFLICT);
-  INT_CONSTANT (IPP_OK_IGNORED_SUBSCRIPTIONS);
-  INT_CONSTANT (IPP_OK_IGNORED_NOTIFICATIONS);
-  INT_CONSTANT (IPP_OK_TOO_MANY_EVENTS);
-  INT_CONSTANT (IPP_OK_BUT_CANCEL_SUBSCRIPTION);
-  INT_CONSTANT (IPP_OK_EVENTS_COMPLETE);
-  INT_CONSTANT (IPP_REDIRECTION_OTHER_SITE);
-  INT_CONSTANT (IPP_BAD_REQUEST);
-  INT_CONSTANT (IPP_FORBIDDEN);
-  INT_CONSTANT (IPP_NOT_AUTHENTICATED);
-  INT_CONSTANT (IPP_NOT_AUTHORIZED);
-  INT_CONSTANT (IPP_NOT_POSSIBLE);
-  INT_CONSTANT (IPP_TIMEOUT);
-  INT_CONSTANT (IPP_NOT_FOUND);
-  INT_CONSTANT (IPP_GONE);
-  INT_CONSTANT (IPP_REQUEST_ENTITY);
-  INT_CONSTANT (IPP_REQUEST_VALUE);
-  INT_CONSTANT (IPP_DOCUMENT_FORMAT);
-  INT_CONSTANT (IPP_ATTRIBUTES);
-  INT_CONSTANT (IPP_URI_SCHEME);
-  INT_CONSTANT (IPP_CHARSET);
-  INT_CONSTANT (IPP_CONFLICT);
-  INT_CONSTANT (IPP_COMPRESSION_NOT_SUPPORTED);
-  INT_CONSTANT (IPP_COMPRESSION_ERROR);
-  INT_CONSTANT (IPP_DOCUMENT_FORMAT_ERROR);
-  INT_CONSTANT (IPP_DOCUMENT_ACCESS_ERROR);
-  INT_CONSTANT (IPP_ATTRIBUTES_NOT_SETTABLE);
-  INT_CONSTANT (IPP_IGNORED_ALL_SUBSCRIPTIONS);
-  INT_CONSTANT (IPP_TOO_MANY_SUBSCRIPTIONS);
-  INT_CONSTANT (IPP_IGNORED_ALL_NOTIFICATIONS);
-  INT_CONSTANT (IPP_PRINT_SUPPORT_FILE_NOT_FOUND);
-  INT_CONSTANT (IPP_INTERNAL_ERROR);
-  INT_CONSTANT (IPP_OPERATION_NOT_SUPPORTED);
-  INT_CONSTANT (IPP_SERVICE_UNAVAILABLE);
-  INT_CONSTANT (IPP_VERSION_NOT_SUPPORTED);
-  INT_CONSTANT (IPP_DEVICE_ERROR);
-  INT_CONSTANT (IPP_TEMPORARY_ERROR);
-  INT_CONSTANT (IPP_NOT_ACCEPTING);
-  INT_CONSTANT (IPP_PRINTER_BUSY);
-  INT_CONSTANT (IPP_ERROR_JOB_CANCELLED);
-  INT_CONSTANT (IPP_MULTIPLE_JOBS_NOT_SUPPORTED);
-  INT_CONSTANT (IPP_PRINTER_IS_DEACTIVATED);
-  INT_CONSTANT (IPP_AUTHENTICATION_CANCELED);
-  INT_CONSTANT (IPP_PKI_ERROR);
+  /* Also define legacy names */
+#if CUPS_VERSION_MAJOR > 1 || (CUPS_VERSION_MAJOR == 1 && CUPS_VERSION_MINOR >= 6)
+#  define INT_IPP_STATUS_ERROR_CONSTANT(name)	\
+  INT_CONSTANT_ALIAS(IPP_STATUS_ERROR_##name, "IPP_"#name)
+#  define INT_IPP_STATUS_OK_CONSTANT(name)	\
+  INT_CONSTANT_ALIAS(IPP_STATUS_##name, "IPP_"#name)
+#else /* CUPS < 1.6 */
+#  define INT_IPP_STATUS_ERROR_CONSTANT(name)	\
+  INT_CONSTANT_ALIAS(IPP_##name, "IPP_STATUS_ERROR_"#name)
+#  define INT_IPP_STATUS_OK_CONSTANT(name)	\
+  INT_CONSTANT_ALIAS(IPP_##name, "IPP_STATUS_"#name)
+#endif /* CUPS < 1.6 */
+
+  INT_IPP_STATUS_OK_CONSTANT (OK);
+  INT_16_CONSTANT (IPP_STATUS_OK_IGNORED_OR_SUBSTITUTED, IPP_OK_SUBST);
+  INT_16_CONSTANT (IPP_STATUS_OK_CONFLICTING, IPP_OK_CONFLICT);
+  INT_IPP_STATUS_OK_CONSTANT (OK_IGNORED_SUBSCRIPTIONS);
+  INT_IPP_STATUS_OK_CONSTANT (OK_IGNORED_NOTIFICATIONS);
+  INT_IPP_STATUS_OK_CONSTANT (OK_TOO_MANY_EVENTS);
+  INT_IPP_STATUS_OK_CONSTANT (OK_BUT_CANCEL_SUBSCRIPTION);
+  INT_IPP_STATUS_OK_CONSTANT (OK_EVENTS_COMPLETE);
+  INT_16_CONSTANT (IPP_STATUS_REDIRECTION_OTHER_SITE,
+		   IPP_REDIRECTION_OTHER_SITE);
+  INT_IPP_STATUS_ERROR_CONSTANT (BAD_REQUEST);
+  INT_IPP_STATUS_ERROR_CONSTANT (FORBIDDEN);
+  INT_IPP_STATUS_ERROR_CONSTANT (NOT_AUTHENTICATED);
+  INT_IPP_STATUS_ERROR_CONSTANT (NOT_AUTHORIZED);
+  INT_IPP_STATUS_ERROR_CONSTANT (NOT_POSSIBLE);
+  INT_IPP_STATUS_ERROR_CONSTANT (TIMEOUT);
+  INT_IPP_STATUS_ERROR_CONSTANT (NOT_FOUND);
+  INT_IPP_STATUS_ERROR_CONSTANT (GONE);
+  INT_IPP_STATUS_ERROR_CONSTANT (REQUEST_ENTITY);
+  INT_IPP_STATUS_ERROR_CONSTANT (REQUEST_VALUE);
+  INT_16_CONSTANT (IPP_STATUS_ERROR_DOCUMENT_FORMAT_NOT_SUPPORTED,
+		   IPP_DOCUMENT_FORMAT);
+  INT_16_CONSTANT (IPP_STATUS_ERROR_ATTRIBUTES_OR_VALUES, IPP_ATTRIBUTES);
+  INT_IPP_STATUS_ERROR_CONSTANT (URI_SCHEME);
+  INT_IPP_STATUS_ERROR_CONSTANT (CHARSET);
+  INT_16_CONSTANT (IPP_STATUS_ERROR_CONFLICTING, IPP_CONFLICT);
+  INT_IPP_STATUS_ERROR_CONSTANT (COMPRESSION_NOT_SUPPORTED);
+  INT_IPP_STATUS_ERROR_CONSTANT (COMPRESSION_ERROR);
+  INT_IPP_STATUS_ERROR_CONSTANT (DOCUMENT_FORMAT_ERROR);
+  INT_16_CONSTANT (IPP_STATUS_ERROR_DOCUMENT_ACCESS, IPP_DOCUMENT_ACCESS_ERROR);
+  INT_IPP_STATUS_ERROR_CONSTANT (ATTRIBUTES_NOT_SETTABLE);
+  INT_IPP_STATUS_ERROR_CONSTANT (IGNORED_ALL_SUBSCRIPTIONS);
+  INT_IPP_STATUS_ERROR_CONSTANT (TOO_MANY_SUBSCRIPTIONS);
+  INT_IPP_STATUS_ERROR_CONSTANT (IGNORED_ALL_NOTIFICATIONS);
+  INT_IPP_STATUS_ERROR_CONSTANT (PRINT_SUPPORT_FILE_NOT_FOUND);
+  INT_16_CONSTANT (IPP_STATUS_ERROR_INTERNAL, IPP_INTERNAL_ERROR);
+  INT_IPP_STATUS_ERROR_CONSTANT (OPERATION_NOT_SUPPORTED);
+  INT_IPP_STATUS_ERROR_CONSTANT (SERVICE_UNAVAILABLE);
+  INT_IPP_STATUS_ERROR_CONSTANT (VERSION_NOT_SUPPORTED);
+  INT_16_CONSTANT (IPP_STATUS_ERROR_DEVICE, IPP_DEVICE_ERROR);
+  INT_16_CONSTANT (IPP_STATUS_ERROR_TEMPORARY, IPP_TEMPORARY_ERROR);
+  INT_16_CONSTANT (IPP_STATUS_ERROR_NOT_ACCEPTING_JOBS, IPP_NOT_ACCEPTING);
+  INT_16_CONSTANT (IPP_STATUS_ERROR_BUSY, IPP_PRINTER_BUSY);
+  INT_16_CONSTANT (IPP_STATUS_ERROR_JOB_CANCELED, IPP_ERROR_JOB_CANCELED);
+  INT_IPP_STATUS_ERROR_CONSTANT (MULTIPLE_JOBS_NOT_SUPPORTED);
+  INT_IPP_STATUS_ERROR_CONSTANT (PRINTER_IS_DEACTIVATED);
+  INT_16_CONSTANT (IPP_STATUS_ERROR_CUPS_AUTHENTICATION_CANCELED,
+		   IPP_AUTHENTICATION_CANCELED);
+  INT_16_CONSTANT (IPP_STATUS_ERROR_CUPS_PKI, IPP_PKI_ERROR);
+#if CUPS_VERSION_MAJOR > 1 || (CUPS_VERSION_MAJOR == 1 && CUPS_VERSION_MINOR >= 5)
+  INT_16_CONSTANT (IPP_STATUS_ERROR_CUPS_UPGRADE_REQUIRED,
+		   IPP_UPGRADE_REQUIRED);
+#endif /* CUPS >= 1.5 */
 
   // IPP states
-  INT_CONSTANT (IPP_ERROR);
-  INT_CONSTANT (IPP_IDLE);
-  INT_CONSTANT (IPP_HEADER);
-  INT_CONSTANT (IPP_ATTRIBUTE);
-  INT_CONSTANT (IPP_DATA);
+  /* Also define legacy names */
+#if CUPS_VERSION_MAJOR > 1 || (CUPS_VERSION_MAJOR == 1 && CUPS_VERSION_MINOR >= 6)
+#  define INT_IPP_STATE_CONSTANT(name)	\
+  INT_CONSTANT_ALIAS(IPP_STATE_##name, "IPP_"#name)
+#else /* CUPS < 1.6 */
+#  define INT_IPP_STATE_CONSTANT(name)		\
+  INT_CONSTANT_ALIAS(IPP_##name, "IPP_STATE_"#name)
+#endif /* CUPS < 1.6 */
+
+  INT_IPP_STATE_CONSTANT (ERROR);
+  INT_IPP_STATE_CONSTANT (IDLE);
+  INT_IPP_STATE_CONSTANT (HEADER);
+  INT_IPP_STATE_CONSTANT (ATTRIBUTE);
+  INT_IPP_STATE_CONSTANT (DATA);
 
   // IPP attribute tags
   INT_CONSTANT (IPP_TAG_ZERO);
@@ -1259,6 +1386,91 @@ initcups (void)
   INT_CONSTANT (IPP_TAG_LANGUAGE);
   INT_CONSTANT (IPP_TAG_MIMETYPE);
 
+  // IPP operations
+#if CUPS_VERSION_MAJOR > 1 || (CUPS_VERSION_MAJOR == 1 && CUPS_VERSION_MINOR >= 6)
+#  define INT_IPP_OP_CONSTANT(name)		\
+  INT_CONSTANT(IPP_OP_##name)
+#else /* CUPS < 1.6 */
+#  define INT_IPP_OP_CONSTANT(name)					\
+  INT_CONSTANT_AS(IPP_##name, "IPP_OP_"#name)
+#endif /* CUPS < 1.6 */
+
+  INT_IPP_OP_CONSTANT (PRINT_JOB);
+  INT_IPP_OP_CONSTANT (PRINT_URI);
+  INT_IPP_OP_CONSTANT (VALIDATE_JOB);
+  INT_IPP_OP_CONSTANT (CREATE_JOB);
+  INT_IPP_OP_CONSTANT (SEND_DOCUMENT);
+  INT_IPP_OP_CONSTANT (SEND_URI);
+  INT_IPP_OP_CONSTANT (CANCEL_JOB);
+  INT_IPP_OP_CONSTANT (GET_JOB_ATTRIBUTES);
+  INT_IPP_OP_CONSTANT (GET_JOBS);
+  INT_IPP_OP_CONSTANT (GET_PRINTER_ATTRIBUTES);
+  INT_IPP_OP_CONSTANT (HOLD_JOB);
+  INT_IPP_OP_CONSTANT (RELEASE_JOB);
+  INT_IPP_OP_CONSTANT (RESTART_JOB);
+  INT_IPP_OP_CONSTANT (PAUSE_PRINTER);
+  INT_IPP_OP_CONSTANT (RESUME_PRINTER);
+  INT_IPP_OP_CONSTANT (PURGE_JOBS);
+  INT_IPP_OP_CONSTANT (SET_PRINTER_ATTRIBUTES);
+  INT_IPP_OP_CONSTANT (SET_JOB_ATTRIBUTES);
+  INT_IPP_OP_CONSTANT (GET_PRINTER_SUPPORTED_VALUES);
+  INT_16_CONSTANT (IPP_OP_CREATE_PRINTER_SUBSCRIPTIONS,
+		   IPP_CREATE_PRINTER_SUBSCRIPTION);
+  INT_16_CONSTANT (IPP_OP_CREATE_JOB_SUBSCRIPTIONS,
+		   IPP_CREATE_JOB_SUBSCRIPTION);
+  INT_IPP_OP_CONSTANT (GET_SUBSCRIPTIONS);
+  INT_IPP_OP_CONSTANT (RENEW_SUBSCRIPTION);
+  INT_IPP_OP_CONSTANT (CANCEL_SUBSCRIPTION);
+  INT_IPP_OP_CONSTANT (GET_NOTIFICATIONS);
+  INT_IPP_OP_CONSTANT (SEND_NOTIFICATIONS);
+#if CUPS_VERSION_MAJOR > 1 || (CUPS_VERSION_MAJOR == 1 && CUPS_VERSION_MINOR >= 6)
+  INT_IPP_OP_CONSTANT (GET_RESOURCE_ATTRIBUTES);
+  INT_IPP_OP_CONSTANT (GET_RESOURCE_DATA);
+  INT_IPP_OP_CONSTANT (GET_RESOURCES);
+#endif /* CUPS >= 1.6 */
+  INT_IPP_OP_CONSTANT (GET_PRINT_SUPPORT_FILES);
+  INT_IPP_OP_CONSTANT (ENABLE_PRINTER);
+  INT_IPP_OP_CONSTANT (DISABLE_PRINTER);
+  INT_IPP_OP_CONSTANT (PAUSE_PRINTER_AFTER_CURRENT_JOB);
+  INT_IPP_OP_CONSTANT (HOLD_NEW_JOBS);
+  INT_IPP_OP_CONSTANT (RELEASE_HELD_NEW_JOBS);
+  INT_IPP_OP_CONSTANT (DEACTIVATE_PRINTER);
+  INT_IPP_OP_CONSTANT (ACTIVATE_PRINTER);
+  INT_IPP_OP_CONSTANT (RESTART_PRINTER);
+  INT_IPP_OP_CONSTANT (SHUTDOWN_PRINTER);
+  INT_IPP_OP_CONSTANT (STARTUP_PRINTER);
+  INT_IPP_OP_CONSTANT (REPROCESS_JOB);
+  INT_IPP_OP_CONSTANT (CANCEL_CURRENT_JOB);
+  INT_IPP_OP_CONSTANT (SUSPEND_CURRENT_JOB);
+  INT_IPP_OP_CONSTANT (RESUME_JOB);
+  INT_IPP_OP_CONSTANT (PROMOTE_JOB);
+  INT_IPP_OP_CONSTANT (SCHEDULE_JOB_AFTER);
+#if CUPS_VERSION_MAJOR > 1 || (CUPS_VERSION_MAJOR == 1 && CUPS_VERSION_MINOR >= 6)
+  INT_IPP_OP_CONSTANT (CANCEL_JOBS);
+  INT_IPP_OP_CONSTANT (CANCEL_MY_JOBS);
+  INT_IPP_OP_CONSTANT (RESUBMIT_JOB);
+  INT_IPP_OP_CONSTANT (CLOSE_JOB);
+  INT_IPP_OP_CONSTANT (IDENTIFY_PRINTER);
+  INT_IPP_OP_CONSTANT (VALIDATE_DOCUMENT);
+  INT_IPP_OP_CONSTANT (SEND_HARDCOPY_DOCUMENT);
+#endif /* CUPS >= 1.6 */
+  INT_16_CONSTANT_NEWNAME (IPP_OP_CUPS_GET_DEFAULT, CUPS_GET_DEFAULT);
+  INT_16_CONSTANT_NEWNAME (IPP_OP_CUPS_GET_PRINTERS, CUPS_GET_PRINTERS);
+  INT_16_CONSTANT_NEWNAME (IPP_OP_CUPS_ADD_MODIFY_PRINTER,
+			   CUPS_ADD_MODIFY_PRINTER);
+  INT_16_CONSTANT_NEWNAME (IPP_OP_CUPS_DELETE_PRINTER, CUPS_DELETE_PRINTER);
+  INT_16_CONSTANT_NEWNAME (IPP_OP_CUPS_GET_CLASSES, CUPS_GET_CLASSES);
+  INT_16_CONSTANT_NEWNAME (IPP_OP_CUPS_ADD_MODIFY_CLASS, CUPS_ADD_MODIFY_CLASS);
+  INT_16_CONSTANT_NEWNAME (IPP_OP_CUPS_DELETE_CLASS, CUPS_DELETE_CLASS);
+  INT_16_CONSTANT_NEWNAME (IPP_OP_CUPS_ACCEPT_JOBS, CUPS_ACCEPT_JOBS);
+  INT_16_CONSTANT_NEWNAME (IPP_OP_CUPS_REJECT_JOBS, CUPS_REJECT_JOBS);
+  INT_16_CONSTANT_NEWNAME (IPP_OP_CUPS_SET_DEFAULT, CUPS_SET_DEFAULT);
+  INT_16_CONSTANT_NEWNAME (IPP_OP_CUPS_GET_PPDS, CUPS_GET_PPDS);
+  INT_16_CONSTANT_NEWNAME (IPP_OP_CUPS_MOVE_JOB, CUPS_MOVE_JOB);
+  INT_16_CONSTANT_NEWNAME (IPP_OP_CUPS_AUTHENTICATE_JOB, CUPS_AUTHENTICATE_JOB);
+  INT_16_CONSTANT_NEWNAME (IPP_OP_CUPS_GET_PPD, CUPS_GET_PPD);
+  INT_16_CONSTANT_NEWNAME (IPP_OP_CUPS_GET_DOCUMENT, CUPS_GET_DOCUMENT);
+
   // Limits
   INT_CONSTANT (IPP_MAX_NAME);
 
